/* PROGRAM: Simple Heat
 * AUTHOR: Aaron Weeden (Shodor Education Foundation)
 * DATE: January 2014
 *
 * See the README for instructions on compiling and running this program.
 */

#include <stdio.h>
#include <errno.h>
#include <getopt.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

int main() {

    /* Data Structure (what the model has) */
    int timeStepCount = 3;
    int rowCount = 3;
    int columnCount = 3;
    float initialTemperature = 100.0;
    typedef struct {
        float temperatureNow;
        float temperatureNext;
        int neighborCount;
    } point_t;
    int pointCount;
    point_t *points;
    int pointIndex;
    int timeStepIndex;
    int option;

    /* Algorithm (how the model computes) */

    /* 1: calculate number of points */
    pointCount = rowCount * columnCount;

    /* allocate memory */
    points = (point_t*)malloc(pointCount * sizeof(point_t));

    for (
            /* 2: start at point 0 */
            pointIndex = 0;

            /* 3: is point outside the object? */
            pointIndex < pointCount;

            /* 10: move on to the next point */
            pointIndex++
        ) {

        /* 4: set initial temperature */
        if (pointIndex % columnCount == 0) {
            points[pointIndex].temperatureNow = initialTemperature;
        } else {
            points[pointIndex].temperatureNow = 0.0;
        }

        /* 5: assume 4 neighbors */
        points[pointIndex].neighborCount = 4;

        /* 6: top row has 1 less neighbor */
        if (pointIndex < columnCount) {
            points[pointIndex].neighborCount--;
        }

        /* 7: bottom row has 1 less neighbor */
        if (pointIndex >= (rowCount-1) * columnCount) {
            points[pointIndex].neighborCount--;
        }

        /* 8: left column has 1 less neighbor */
        if (pointIndex % columnCount == 0) {
            points[pointIndex].neighborCount--;
        }

        /* 9: right column has 1 less neighbor */
        if (pointIndex % columnCount == columnCount-1) {
            points[pointIndex].neighborCount--;
        }
    }

    for (
            /* 11: start at time step 0 */
            timeStepIndex = 0;

            /* 12: are we finished? */
            timeStepIndex < timeStepCount;

            /* 32: move on to the next time step */
            timeStepIndex++
        ) {

        for (
                /* 13: start at point 0 */
                pointIndex = 0;

                /* 14: is point outside the object? */
                pointIndex < pointCount;

                /* 17: move on to the next point */
                pointIndex++
            ) {

            /* 15: print temperature */
            printf("%.2f ", points[pointIndex].temperatureNow);

            /* 16: print newline between rows */
            if (pointIndex % columnCount == columnCount-1) {
                printf("\n");
            }
        }

        /* 18: print newline between time steps */
        printf("\n");

        for (
                /* 19: start at point 0 */
                pointIndex = 0;

                /* 20: is point outside the object? */
                pointIndex < pointCount;

                /* 27: move on to the next point */
                pointIndex++
            ) {

            /* 21: initialize next temperature */
            points[pointIndex].temperatureNext = 0.0;

            /* 22: add top neighbor */
            if (pointIndex >= columnCount) {
                points[pointIndex].temperatureNext +=
                    points[pointIndex-columnCount].temperatureNow;
            }

            /* 23: add bottom neighbor */
            if (pointIndex < (rowCount-1) * columnCount) {
                points[pointIndex].temperatureNext +=
                    points[pointIndex+columnCount].temperatureNow;
            }

            /* 24: add left neighbor */
            if (pointIndex % columnCount > 0) {
                points[pointIndex].temperatureNext +=
                    points[pointIndex-1].temperatureNow;
            }

            /* 25: add right neighbor */
            if (pointIndex % columnCount < columnCount-1) {
                points[pointIndex].temperatureNext +=
                    points[pointIndex+1].temperatureNow;
            }

            /* 26: divide by number of neighbors */
            points[pointIndex].temperatureNext =
                points[pointIndex].temperatureNext /
                points[pointIndex].neighborCount;
        }

        for (
                /* 28: start at point 0 */
                pointIndex = 0;

                /* 29: is point outside the object? */
                pointIndex < pointCount;

                /* 31: move on to the next point */
                pointIndex++
            ) {

            /* 30: advance temperature */
            points[pointIndex].temperatureNow =
                points[pointIndex].temperatureNext;
        }
    }

    /* deallocate memory */
    free(points);

    return 0;
}
